#define STARTUP_TITLE	"Timer "
//***************************************************************************
//
//  File........: main.c
//
//  Author(s)...: ATMEL Norway
//
//  Target(s)...: ATmega169
//
//  Compiler....: AVR-GCC 3.3.1; avr-libc 1.0
//
//  Description.: AVR Butterfly main module
//
//  Revisions...: 1.0
//
//  YYYYMMDD - VER. - COMMENT                                       	- SIGN.
//
//  20030116 - 1.0  - Created                                       	- KS
//  20031009          port to avr-gcc/avr-libc                      	- M.Thomas (*)
//  20031204          fixed imcompatibility with sleep-modes        	- mt
//  20040218          fixed 'logical and' in calibration            	- shc/mt
//  20050827          fixed avr-libc iom169.h compatibility
//                    added keyklick function (from version6)       	- mt/v6
//  20051017          DSLR-Timer based on AVR Butterfly eval-code		- Michael P.
//
//  20060429 - 1.1  - Work started for enhanced GUI (blinking char) 	- Thomas Knoblauch
//                  - Functionality improved: NEW: 9999 counts possible - Thomas Knoblauch
//           - 1.4  - Some optimation in code size done              	- Thomas Knoblauch
//  20060526 - 1.5  - DRI Function added 	                            - Thomas Knoblauch
//
//
//***************************************************************************

// (*) Martin Thomas, Kaiserslautern, Germany, e-mail: mthomas(at)rhrk.uni-kl.de 
// or eversmith(at)heizung-thomas.de
//
// I'm not working for ATMEL.
// The port is based on REV_06 of the ATMEL-Code (for IAR-C)
// Initialy I marked my changes with "// mt" or enclosed them with "// mtA" and 
// "// mtE" but forgot this for some changes esp. during debugging. 'diff' 
// against the original code to see everything that has been changed.

// 17. october 2005
// Michael Posavec, Vienna, Austria, e-mail: lateralusmail(at)yahoo.de
// This code is based on the gcc port from Martin Thomas, and I'm not working for
// ATMEL either.
// I didn't want to spend any money buying an expensive timer-remote-control for my
// DSLR, thus I searched for a development board with sepecific features
// (available module including LCD, some IO's, some buttons, free development tools,
// no additional hardware eg.: JTAG-box needed for flashing of code) and found the
// perfect solution in ATMEL's AVR butterfly demo board. Using the code below in
// conjunction with the AVR butterfly you just have to upload the code via RS232
// to the board and you can use PB2 on J400 as shutter-signal for your Canon EOS-DSLR.
// (GND is also available on J400)

// 29. April 2006
// Thomas Knoblauch, Switzerland, Homepage: http://avila.star-shine.ch (for more info about me).
// I'm not working for ATMEL either.
// Work started for improofing the GUI. The goal is to have a more intuitive GUI and extended
// function: Bigger counts and DRI function.




//mtA
//#include <inavr.h>
//#include "iom169.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <inttypes.h>
//mtE

#include "main.h"
#include "LCD_functions.h"
#include "LCD_driver.h"
#include "button.h"
#include "RTC.h"
#include "timer0.h"
#include "BCD.h"
#include "sound.h"
#include "ADC.h"
#include "dataflash.h"
#include "eeprom169.h"

//#include "eeprom.h"
// mt Test() is not realy needed 
//- can not be accessed without external hardware
#include "test.h"	
#include "vcard.h"
//#include "menu.h"

#define pLCDREG_test (*(char *)(0xEC))


#define AF_ON			(PORTB &= ~0x01)
#define AF_OFF			(PORTB |= 0x01)

#define SHUTTER_ON		(PORTB &= ~0x04)
#define SHUTTER_OFF		(PORTB |= 0x04)

#define LED_ON			(PORTB &= ~0x08)
#define LED_OFF			(PORTB |= 0x08)

#define LED_ON_TIME		3

//mt: extern __flash unsigned int LCD_character_table[];
//    but this is not used here anyway...
extern unsigned int LCD_character_table[] PROGMEM;

extern volatile uint16_t gPowerSaveTimer;    	// external Counter from "RTC.c"
#define  POWER_SAVE_TIMEOUT 250     			// Initial value, enable power save mode after 300 sec

BOOL KeyClickStatus = FALSE;    // Variable to enable/disable keyclick

char gAutoPressJoystick = FALSE;// global variable used in "LCD_driver.c"


unsigned char state;            // helds the current state, according to 
                                // "menu.h"
int gShootMin1, gShootMin10, gShootSec1, gShootSec10, gWaitMin1, gWaitMin10 ,gWaitSec1, gWaitSec10;
int gShootCount1, gShootCount10, gShootCount100, gShootCount1000;
int  gLED, gLEDtimer;
long gShootCountTemp;
char gClick,gBuzzer;
int gDRIMin, gDRIMax, gDRIStep;	// TK:  for DRI functionality
int i=0;							// TK:  count variable
int blinktime = 1;					// TK:  Defines the time for blinking character, to show where the cursor is
int gCameraWait = 1000;				// FJB: Wait for Camera (AF)
int gMirrorTime = 5000;				// FJB: Mirrortime before Shutter

// TK: Variables needed for DRI functions
long waitBetweenImages = 1000;		// miliseconds

/*
char s0[]	= "1/1000";
char s1[]	= "1/500 ";
char s2[]	= "1/250 ";
char s3[]	= "1/125 ";
char s4[]	= "1/60  ";
char s5[]	= "1/30  ";
char s6[]	= "1/15  ";
char s7[]	= "1/8   ";
char s8[]	= "1/4   ";
char s9[]	= "1/2   ";
char s10[]	= " 1 SEC";
char s11[]	= " 2 SEC";
char s12[]	= " 4 SEC";
char s13[]	= " 8 SEC";
char s14[]	= "15 SEC";
char s15[]	= "30 SEC";
char s16[]	= " 1 MIN";
char s17[]	= " 2 MIN";
char s18[]	= " 4 MIN";

char * pShutterSpeed[] = {s0,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13,s14,s15,s16,s17,s18};
// miliseconds
unsigned long ShutterTime[] = {1,2,4,8,16,33,66,125,250,500,1000,2000,4000,8000,15000,30000,60000,120000,240000};
*/

char * pShutterSpeed[] =
{
"1/250 ", "1/200 ", "1/160 ",
"1/125 ", "1/100 ", "1/80  ",
"1/60  ", "1/50  ", "1/40  ",
"1/30  ", "1/25  ", "1/20  ",
"1/15  ", "1/13  ", "1/10  ",
"1/8   ", "1/6   ", "1/5   ",
"1/4   ", " 0,3 S", " 0,4 S",
" 0,5 S", " 0,6 S", " 0,8 S",
" 1 SEC", " 1,3 S", " 1,6 S",
" 2 SEC", " 2,5 S", " 3,2 S",
" 4 SEC", " 5 SEC", " 6 SEC",
" 8 SEC", "10 SEC", "13 SEC",
"15 SEC", "20 SEC", "25 SEC",
"30 SEC", "40 SEC", "50 SEC",
" 1 MIN", " 1,3 M", " 1,6 M",
" 2 MIN", " 2,5 M", " 3,2 M",
" 4 MIN", " 5 MIN", " 6 MIN"
};
// miliseconds
unsigned long ShutterTime[] =
{
4,		5,		6,
8,		10,		13,
17,		20,		25,		
33,		40,		50,
66,		77,		100,
125,	166,	200,
250,	333,	400,
500,	600,	800,
1000,	1300,	1600,
2000,	2500,	3200,
4000,	5000,	6000,
8000,	10000,	13000,
15000,	20000,	25000,
30000,	40000,	50000,
60000,	80000,	100000,
120000,	150000,	192000,
240000,	300000,	360000
};

int NumberOfTimes = sizeof (ShutterTime)/sizeof (unsigned long);

// FJB: Selections for Camera and Mirror Timer
char *			pMirrorTime[] = { "   OFF", " 2 SEC", " 5 SEC", "10 SEC", "15 SEC", "20 SEC" };
unsigned long	MirrorTime[]  = { 0, 2000, 5000, 10000, 15000, 20000 };
int				NumberOfMirror= sizeof (MirrorTime)/sizeof (unsigned long);

char *			pCameraWait[] = { "   OFF", "   1/2", " 1 SEC", " 1 1/2", " 2 SEC", " 2 1/2" };
unsigned long	CameraWait[]  = { 0, 500, 1000, 1500, 2000, 2500 };
int				NumberOfCamera= sizeof (CameraWait)/sizeof (unsigned long);


/*****************************************************************************
*
*   Function name : main
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Contains the main loop of the program
*
*****************************************************************************/
// mt __C_task void main(void)
int main(void)
{    
//    unsigned char state, nextstate;
    char input;
	int ADC_value;
	int ADC_value_backup;
	unsigned char minH,minL,secH,secL;


    // Program initalization
    Initialization();
    sei(); // mt __enable_interrupt();

	//KeyClickStatus=TRUE;
	//ADMUX = TEMPERATURE_SENSOR;
	ADMUX = LIGHT_SENSOR;
	
	ADC_init(LIGHT_SENSOR);
    ADC_value = ADC_read();         // Read the light value
	ADC_value_backup = ADC_value;	// FJB: Synchronize ADC-Backup value
	
	state=ST_DEFAULT;
	gShootSec1=eeprom_read_byte_169(E2PROM_SHOOTSEC_1);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gShootSec1 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootSec1 =0;

	gShootSec10=eeprom_read_byte_169(E2PROM_SHOOTSEC_10);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gShootSec10 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootSec10 =0;

	gShootMin1=eeprom_read_byte_169(E2PROM_SHOOTMIN_1);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gShootMin1 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootMin1 =0;

	gShootMin10=eeprom_read_byte_169(E2PROM_SHOOTMIN_10);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gShootMin10 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootMin10 =0;

	gWaitSec1=eeprom_read_byte_169(E2PROM_WAITSEC_1);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitSec1 == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitSec1 =0;

	gWaitSec10=eeprom_read_byte_169(E2PROM_WAITSEC_10);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitSec10 == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitSec10 =0;

	gWaitMin1=eeprom_read_byte_169(E2PROM_WAITMIN_1);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitMin1 == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitMin1 =0;

	gWaitMin10=eeprom_read_byte_169(E2PROM_WAITMIN_10);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitMin10 == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitMin10 =0;

	gShootCount1=eeprom_read_byte_169(E2PROM_SHOOTCOUNT_1);	// read out the shooting counter from EEPROM
	// empty E2PROM has every single bit set
	if (gShootCount1 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootCount1 =0;

	// TK: need for expanding to 9999 counts
	gShootCount10=eeprom_read_byte_169(E2PROM_SHOOTCOUNT_10);	// read out the shooting counter from EEPROM
	// empty E2PROM has every single bit set
	if (gShootCount10 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootCount10 =0;

	gShootCount100=eeprom_read_byte_169(E2PROM_SHOOTCOUNT_100);	// read out the shooting counter from EEPROM
	// empty E2PROM has every single bit set
	if (gShootCount100 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootCount100 =0;

	gShootCount1000=eeprom_read_byte_169(E2PROM_SHOOTCOUNT_1000);	// read out the shooting counter from EEPROM
	// empty E2PROM has every single bit set
	if (gShootCount1000 == 0xFF)	// if this is the very first time, reset the value to 0
		gShootCount1000 =0;

	gClick=eeprom_read_byte_169(E2PROM_CLICK);	// read out the click setting from EEPROM
	// empty E2PROM has every single bit set
	if (gClick == 0xFF)	// if this is the very first time, reset the value to 0
		gClick =0;
	KeyClickStatus= gClick;		// update the status

	gLED=eeprom_read_byte_169(E2PROM_LED);	// read out the LED setting from EEPROM
	// empty E2PROM has every single bit set
	if (gLED == 0xFF)	// if this is the very first time, reset the value to 0
		gLED =LED_STATE_OFF;

	gBuzzer=eeprom_read_byte_169(E2PROM_BUZZER);	// read out the Buzzer setting from EEPROM
	// empty E2PROM has every single bit set
	if (gBuzzer == 0xFF)	// if this is the very first time, reset the value to 0
		gBuzzer = 0;

	// For DRI Functionality
	gDRIMin=eeprom_read_byte_169(E2PROM_DRI_MIN);	// read out the DRI min Shutter setting from EEPROM
	// empty E2PROM has every single bit set
	if (gDRIMin == 0xFF)	// if this is the very first time, reset the value to 0
		gDRIMin = 0;

	gDRIMax=eeprom_read_byte_169(E2PROM_DRI_MAX);	// read out the DRI max Shutter setting from EEPROM
	// empty E2PROM has every single bit set
	if (gDRIMax >= NumberOfTimes)	// if this is the very first time, reset the value to 0
		gDRIMax = NumberOfTimes-1;

	gDRIStep=eeprom_read_byte_169(E2PROM_DRI_STEP);	// read out the DRI Step for Shutter setting from EEPROM
	// empty E2PROM has every single bit set
	if ((gDRIStep != 1) && (gDRIStep != 3))	// if this is the very first time, reset the value to 0
		gDRIStep = 3;

	// For MIRROR Functionality
	gMirrorTime=eeprom_read_byte_169(E2PROM_MIRROR_TIME);	// read out the Mirror setting from EEPROM
	// empty E2PROM has every single bit set
	if (gMirrorTime >= NumberOfMirror)	// if this is the very first time, reset the value to 0
		gMirrorTime = 0;

	// For CAMERA Functionality
	gCameraWait=eeprom_read_byte_169(E2PROM_CAMERA_WAIT);	// read out the Cameratime (AF Time) setting from EEPROM
	// empty E2PROM has every single bit set
	if (gCameraWait >= NumberOfCamera)	// if this is the very first time, reset the value to 0
		gCameraWait = 0;



	Sound_Init();

	while(1)
	{	
		input = getkey();           // Read buttons
		if (gPowerSaveTimer>=POWER_SAVE_TIMEOUT)
		{
			// return to bootloader in power save mode -> this draws only 8,5 uA from the battery
    	    WDTCR = (1<<WDCE) | (1<<WDE);     //Enable Watchdog Timer to give reset
   	    	while(1);   	// wait for watchdog-reset, since the BOOTRST-fuse is 
		                    	// programmed, the Boot-section will be entered upon reset
		}

		// if LED is turned off all the time
		if (gLED == LED_STATE_OFF)
			LED_OFF;

		// if LED is turned on all the time
		if (gLED == LED_STATE_ON)
			LED_ON;

		// if LED is turned on only at night, and turned off during the day
		ADMUX = LIGHT_SENSOR;
		ADC_init(LIGHT_SENSOR);
	    ADC_value = ADC_read();         // Read the light value
		if (gLED == LED_STATE_LIGHT)
		{
//			ADMUX = LIGHT_SENSOR;
//			ADC_init(LIGHT_SENSOR);
//		    ADC_value = ADC_read();         // Read the light value

			if (ADC_value > 0x3E8)
				LED_ON;						// turn LED on during night
			if (ADC_value < 0x3D8)
				LED_OFF;					// turn LED off during day
		}
		
		// if LED is turned for LED_ON_TIME seconds after any keypress event and
		// LED_ON_TIME seconds before expiration of the shooting- or wait-time
		if (gLED == LED_STATE_AUTO)
		{
			if (input != KEY_NULL)
				gLEDtimer=LED_ON_TIME;		// start the LED timer if keypress event

			if (gLEDtimer)				
				LED_ON;						// turn on LED if timer is running
			else
				LED_OFF;					// turn off ohterwise
		}

/************* DEFAULT STATE *****************/
		if (state == ST_DEFAULT)
		{
			// the default state with its "TIMER" LCD-output is only displayed once during system boot
			// this state is never entered again
		    LCD_puts_f(PSTR(STARTUP_TITLE),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_OPTION;
			if (input == KEY_MINUS)
				state = ST_AUTO;
		}
/************* AUTO STATE ********************/
		else if (state == ST_AUTO)
		{
		    LCD_puts_f(PSTR("AUTO"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_OPTION;		// skip over default state since this is only
										// the welcome screen "TIMER"
			if (input == KEY_MINUS)
				state = ST_DRI;

			if (input == KEY_NEXT)
				state = ST_AUTO_START;
		}

/************* DRI STATE ********************/
		else if (state == ST_DRI)
		{
		    LCD_puts_f(PSTR("DRI"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_AUTO;
			if (input == KEY_MINUS)
				state = ST_LIGHT;

			if (input == KEY_NEXT)
				state = ST_DRI_RUN;
		}

/************* LIGHT STATE ********************/
		else if (state == ST_LIGHT)
		{
		    LCD_puts_f(PSTR("LIGHT"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_DRI;
			if (input == KEY_MINUS)
				state = ST_MAN;

			if (input == KEY_NEXT)
				state = ST_LIGHT_RUN;
		}

/************* MANUAL STATE ******************/
		else if (state == ST_MAN)
		{
		    LCD_puts_f(PSTR("MAN"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_LIGHT;
			if (input == KEY_MINUS)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_MAN_STOP;			
		}
/************* SET STATE *********************/
		else if (state == ST_SET)
		{
		    LCD_puts_f(PSTR("SET"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_MAN;
			if (input == KEY_MINUS)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_SHOOT_TIME;
		}
/************* OPTION STATE ******************/
		else if (state == ST_OPTION)
		{
		    LCD_puts_f(PSTR("OPTION"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SET;
			if (input == KEY_MINUS)		// skip over default state since this is only
				state = ST_AUTO;		// the welcome screen "TIMER"

			if (input == KEY_NEXT)
				state = ST_OPTION_BUZZER;
		
			// enter bootloader if ENTER is pressed while in OPTION menu
			if (input == KEY_ENTER)
			{
	    	    WDTCR = (1<<WDCE) | (1<<WDE);     //Enable Watchdog Timer to give reset
    	    	while(1);   	// wait for watchdog-reset, since the BOOTRST-fuse is 
		                    	// programmed, the Boot-section will be entered upon reset
			}
		}

/* * * * * * * * S U B M E N U s * * * * * * */
/************* AUTO START ********************/
		else if (state == ST_AUTO_START)
		{
		    LCD_puts_f(PSTR("ASTART"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_AUTO;
			
			if (input == KEY_ENTER)
			{
				state = ST_AUTO_RUN_SHOOT;
				gShootCountTemp = 1000*gShootCount1000 + 100*gShootCount100 + 10*gShootCount10 + gShootCount1;		// get the number of cycles
				gSECOND=gShootSec1 + 10*gShootSec10 + 60*(gShootMin1 + 10*gShootMin10);	// calculate shoot time in seconds
				gSECOND++;							// add one second, because camera does not start immediately
				Timer0_Init();						// reset the 1s-timer
			}
		}
/************* AUTO RUN SHOOT ****************/
		else if (state == ST_AUTO_RUN_SHOOT)
		{
			SHUTTER_ON;			// activate shutter	

			LCD_Colon(TRUE);	// turn on the ":" on LCD

		    gPowerSaveTimer = 0;                // Reset the Auto Power Down timer

			// if only LED_ON_TIME seconds are left
			if (gSECOND == LED_ON_TIME)
			{
				gLEDtimer = LED_ON_TIME;				// set the LED timer to turn on the LED for
														// the last LED_ON_TIME seconds.
				// if Buzzer is enabled
				if (gBuzzer)
					TCCR1A = 0xC1;		// turn on fast PWM mode
			}

			// turn off the buzzer after 1s
			if (gSECOND == (LED_ON_TIME-1))
				TCCR1A = 0x00;		// turn off fast PWM mode

			// if shooting time has expired
			if (gSECOND <= 0)
			{
				gShootCountTemp--;						// decrease the number of cycles to come

				// if all the cycles have been executed -> exit auto mode
				if (gShootCountTemp == 0)
				{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
				}
				else
				{
					state = ST_AUTO_RUN_WAIT;			// next state is the wait time
					gSECOND=gWaitSec1 + 10*gWaitSec10 + 60*(gWaitMin1 + 10*gWaitMin10);		// calculate wait time in seconds
				}
			}


			minH = gSECOND / 60;
			secH = gSECOND % 60;

			minH = CHAR2BCD2(minH);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			secH = CHAR2BCD2(secH);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

			DispString('S',' ',minH,minL,secH,secL);	// Displays the string

			// exit at anytime with PREV key
			if (input == KEY_PREV)
			{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
			}

		}
/************* AUTO RUN WAIT *****************/
		else if (state == ST_AUTO_RUN_WAIT)
		{
			SHUTTER_OFF;			// deactivate shutter	

		    gPowerSaveTimer = 0;                // Reset the Auto Power Down timer

			// if shooting time has expired
			if (gSECOND <= 0)
			{
				state = ST_AUTO_RUN_SHOOT;			// next state is again the shoot time
				gSECOND=gShootSec1 + 10*gShootSec10 + 60*(gShootMin1 + 10*gShootMin10);	// calculate shoot time in seconds
				gSECOND++;							// add one second, because camera does not start immediately
				Timer0_Init();						// reset the 1s-timer
			}

			// if only LED_ON_TIME seconds are left
			if (gSECOND == LED_ON_TIME)
				gLEDtimer = LED_ON_TIME;				// set the LED timer to turn on the LED for

			minH = gSECOND / 60;
			secH = gSECOND % 60;

			minH = CHAR2BCD2(minH);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			secH = CHAR2BCD2(secH);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

			DispString('W',' ',minH,minL,secH,secL);	// Displays the string

			// exit at anytime with PREV key
			if (input == KEY_PREV)
			{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
			}
		}
/************* DRI run ********************/
		else if (state == ST_DRI_RUN)
		{
		    LCD_puts_f(PSTR("DRIRUN"),0);	// output string to LCD


			if (gDRIMin > gDRIMax)		//TK: Swap values if they are in oposite order than they should be
			{
				i = gDRIMax;
				gDRIMax = gDRIMin;
				gDRIMin = i;
			}


			for (i=gDRIMin; i<=gDRIMax; i += gDRIStep)
			{
				LCD_puts(pShutterSpeed[i],0);		// Show the shutter time
				gPowerSaveTimer = 0;            	// Reset the Auto Power Down timer

			//	FJB: Wait for AF
				AF_ON;								// FJB: activate AutoFocus
				Delay(CameraWait[gCameraWait]);		// FJB: wait for AutoFocus

			//	FJB: Mirror
				if ( gMirrorTime > 0 )
				{
					LCD_puts("MIRROR", 0);			// FJB: Load Msg "Mirror locked"
					SHUTTER_ON;						// FJB: lock Mirror
					Delay(CameraWait[gCameraWait]);	// FJB: wait to lock Mirror
					SHUTTER_OFF;					// FJB: release Shutterswitch for next operation
					AF_OFF;
					Delay(MirrorTime[gMirrorTime]);	// FJB: wait before Shutter on
				};

			//	FJB: Shutter
				LCD_puts(pShutterSpeed[i],0);		// Show the shutter time
				SHUTTER_ON;							// Shutter on
				Delay(ShutterTime[i]);				// Wait according to ShutterSettings
				SHUTTER_OFF;						// Shutter of
				AF_OFF;								// FJB: release AutoFocus

			//	FJB: Wait for next Image
				LCD_puts("W NEXT", 0);				// FJB: wait for next Image
				waitBetweenImages = 1000 * ( gWaitSec1 + 10*gWaitSec10 + 60*(gWaitMin1 + 10*gWaitMin10) );
				Delay(waitBetweenImages);			// Wait acording to Wait between images

			}
			// Turn on Powersaving	
			// At the end of the Sequence: go to State DRI
			state = ST_DRI;


		}


/************* LIGHT RUN *********************/
		else if (state == ST_LIGHT_RUN)
		{
		    LCD_puts_f(PSTR("LIGRUN"),0);	// output string to LCD

		//	FJB: Wait for Light
			if ( ADC_value > 0x300 )
			{
				AF_ON;							// FJB: activate AutoFocus
				Delay(CameraWait[gCameraWait]);	// FJB: wait for AutoFocus
				SHUTTER_ON;						// FJB: activate shutter	
				Delay(CameraWait[gCameraWait]);	// FJB: wait for Camera
				SHUTTER_OFF;					// FJB: Shutter of
				AF_OFF;							// FJB: release AutoFocus
			};

			if (input == KEY_PREV)
			{
				SHUTTER_OFF;				// deactivate shutter
				AF_OFF;
				state = ST_LIGHT;
			}
			Delay(1000);
		}




/************* MAN STOP **********************/
		else if (state == ST_MAN_STOP)
		{
		    LCD_puts_f(PSTR("MSTOP"),0);	// output string to LCD
			SHUTTER_OFF;					// deactivate shutter	

			if (input == KEY_PREV)
				state = ST_MAN;

			if (input == KEY_ENTER)
				state = ST_MAN_START;
		}

/************* MAN START *********************/
		else if (state == ST_MAN_START)
		{
		    LCD_puts_f(PSTR("MSTART"),0);	// output string to LCD
			SHUTTER_ON;						// activate shutter	

			if (input == KEY_PREV)
			{
				SHUTTER_OFF;				// deactivate shutter	
				state = ST_MAN;
			}

			if (input == KEY_ENTER)
				state = ST_MAN_STOP;
		}




// The submenues in Position SET.
/************* SHOOT TIME ********************/
		else if (state == ST_SHOOT_TIME)
		{
		    LCD_puts_f(PSTR("SHOOT"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_CAMERA_WAIT;
			if (input == KEY_MINUS)
				state = ST_WAIT_TIME;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_SHOOT_TIME_SET_SEC_1;
			
		}
/************* SHOOT TIME SET SEC_1 ************/
		else if (state == ST_SHOOT_TIME_SET_SEC_1)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gShootSec1++;
			if (input == KEY_MINUS)
				gShootSec1--;

			if (gShootSec1 > 9)	// TK: seconds Overflow is handled
			{	
				gShootSec1 = 0;
			}

			if (gShootSec1 < 0)	// TK: seconds Underflow is handled
			{
				gShootSec1 = 9;
			}

			if (input == KEY_PREV)
			{
				state = ST_SHOOT_TIME_SET_SEC_10;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}

					
			DispString('S',' ',gShootMin10+48,gShootMin1+48,gShootSec10+48,' ');	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(5, gShootSec1+48);	// lets the character blink

		}

/************* SHOOT TIME SET SEC_10 ************/
		else if (state == ST_SHOOT_TIME_SET_SEC_10)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gShootSec10++;
			if (input == KEY_MINUS)
				gShootSec10--;

			if (gShootSec10 > 5)	// TK: seconds Overflow is handled
			{	
				gShootSec10 = 0;
			}

			if (gShootSec10 < 0)	// TK: seconds Underflow is handled
			{
				gShootSec10 = 5;
			}

			if (input == KEY_PREV)
			{
				state = ST_SHOOT_TIME_SET_MIN_1;
			}

			if (input == KEY_NEXT)
			{
				state = ST_SHOOT_TIME_SET_SEC_1;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}


			DispString('S',' ',gShootMin10+48,gShootMin1+48,' ',gShootSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(4, gShootSec10+48);	// lets the character blink

		}



/************* SHOOT TIME SET MIN_1 ************/
		else if (state == ST_SHOOT_TIME_SET_MIN_1)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gShootMin1++;
			if (input == KEY_MINUS)
				gShootMin1--;

			if (gShootMin1 > 9)
				gShootMin1 = 0;

			if (gShootMin1 < 0)
				gShootMin1 = 9;

			if (input == KEY_PREV)
			{
				state = ST_SHOOT_TIME_SET_MIN_10;
			}

			if (input == KEY_NEXT)
			{
				state = ST_SHOOT_TIME_SET_SEC_10;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}


			DispString('S',' ',gShootMin10+48,' ',gShootSec10+48,gShootSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(3, gShootMin1+48);	// lets the character blink

		}

/************* SHOOT TIME SET MIN_10 ************/
		else if (state == ST_SHOOT_TIME_SET_MIN_10)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gShootMin10++;
			if (input == KEY_MINUS)
				gShootMin10--;

			if (gShootMin10 > 9)
				gShootMin10 = 0;

			if (gShootMin10 < 0)
				gShootMin10 = 9;

			if (input == KEY_NEXT)
			{
				state = ST_SHOOT_TIME_SET_MIN_1;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}


			DispString('S',' ',' ',gShootMin1+48,gShootSec10+48,gShootSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(2, gShootMin10+48);	// lets the character blink

		}


/************* WAIT TIME *********************/
		else if (state == ST_WAIT_TIME)
		{
		    LCD_puts_f(PSTR("WAIT"),0);		// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SHOOT_TIME;
			if (input == KEY_MINUS)
				state = ST_SHOOT_COUNT;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_WAIT_TIME_SET_SEC_1;
		}

/************* WAIT  TIME SET SEC_1 ************/
		else if (state == ST_WAIT_TIME_SET_SEC_1)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gWaitSec1++;
			if (input == KEY_MINUS)
				gWaitSec1--;

			if (gWaitSec1 > 9)	// TK: seconds: overflow is handeled
			{	gWaitSec1 = 0;
			}

			if (gWaitSec1 < 0)	// TK: seconds: overflow is handeled
			{	gWaitSec1 = 9;
			}

			if (input == KEY_PREV)
			{
				state = ST_WAIT_TIME_SET_SEC_10;
			}

			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
			}


			DispString('W',' ',gWaitMin10+48,gWaitMin1+48,gWaitSec10+48,' ');	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(5, gWaitSec1+48);	// lets the character blink
		}

/************* WAIT  TIME SET SEC_10 ************/
		else if (state == ST_WAIT_TIME_SET_SEC_10)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gWaitSec10++;
			if (input == KEY_MINUS)
				gWaitSec10--;

			if (gWaitSec10 > 5)	// TK: seconds: overflow is handeled
			{	gWaitSec10 = 0;
			}

			if (gWaitSec10 < 0)	// TK: seconds: overflow is handeled
			{	gWaitSec10 = 5;
			}

			if (input == KEY_PREV)
			{
				state = ST_WAIT_TIME_SET_MIN_1;
			}

			if (input == KEY_NEXT)
			{
				state = ST_WAIT_TIME_SET_SEC_1;
			}


			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
			}


			DispString('W',' ',gWaitMin10+48,gWaitMin1+48,' ',gWaitSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(4, gWaitSec10+48);	// lets the character blink
		}


/************* WAIT  TIME SET MIN_1 ************/
		else if (state == ST_WAIT_TIME_SET_MIN_1)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gWaitMin1++;
			if (input == KEY_MINUS)
				gWaitMin1--;

			if (gWaitMin1 > 9)
				gWaitMin1 = 0;

			if (gWaitMin1 < 0)
				gWaitMin1 = 9;

			if (input == KEY_PREV)
			{
				state = ST_WAIT_TIME_SET_MIN_10;
			}

			if (input == KEY_NEXT)
			{
				state = ST_WAIT_TIME_SET_SEC_10;
			}

			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}


			DispString('W',' ',gWaitMin10+48,' ',gWaitSec10+48,gWaitSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(3, gWaitMin1+48);	// lets the character blink
		}

/************* WAIT  TIME SET MIN_10 ************/
		else if (state == ST_WAIT_TIME_SET_MIN_10)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD

			if (input == KEY_PLUS)
				gWaitMin10++;
			if (input == KEY_MINUS)
				gWaitMin10--;

			if (gWaitMin10 > 9)
				gWaitMin10 = 0;

			if (gWaitMin10 < 0)
				gWaitMin10 = 9;


			if (input == KEY_NEXT)
			{
				state = ST_WAIT_TIME_SET_MIN_1;
			}

			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				SaveConfig();		// Save the whole configuration
				}


			DispString('W',' ',' ',gWaitMin1+48,gWaitSec10+48,gWaitSec1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(2, gWaitMin10+48);	// lets the character blink
		}


/************* SHOOT COUNT ********************/
		else if (state == ST_SHOOT_COUNT)
		{
		    LCD_puts_f(PSTR("COUNT"),0);			// output string to LCD

			if (input == KEY_PLUS)
				state = ST_WAIT_TIME;
			if (input == KEY_MINUS)
				state = ST_DRI_SET_MIN;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_SHOOT_COUNT_SET_1;
			
		}
/************* SHOOT COUNT 1********************/
// TK: number of counts increased to 9999
		else if (state == ST_SHOOT_COUNT_SET_1)
		{
	

			if (input == KEY_PLUS)
				gShootCount1++;
			if (input == KEY_MINUS)
				gShootCount1--;

			if (gShootCount1 > 9)		// TK: overflow handling
				gShootCount1 = 0;

			if (gShootCount1 < 0)		// TK: Underflow handling
 				gShootCount1 = 9;

			if (input == KEY_PREV)			// TK: Go to edit the 10s
			{
				state = ST_SHOOT_COUNT_SET_10;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_COUNT;
				SaveConfig();	// Save the whole configuration
			}


			DispString('C',' ',gShootCount1000+48,gShootCount100+48,gShootCount10+48,' ');	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(5, gShootCount1+48);	// lets the character blink

		}

/************* SHOOT COUNT 10********************/  //TK: for faster changeing the 10th
		else if (state == ST_SHOOT_COUNT_SET_10)
		{
			
			if (input == KEY_PLUS)
				gShootCount10++;
			if (input == KEY_MINUS)
				gShootCount10--;

			if (gShootCount10 > 9)		// TK: overflow handling
				gShootCount10 = 0;

			if (gShootCount10 < 0)		// TK: Underflow handling
 				gShootCount10 = 9;

			if (input == KEY_NEXT)			// TK: go to change the 1s
			{	state = ST_SHOOT_COUNT_SET_1;
			}

			if (input == KEY_PREV)			// TK: go to change the 100s
			{	state = ST_SHOOT_COUNT_SET_100;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_COUNT;
				SaveConfig();	// Save the whole configuration

			}


			DispString('C',' ',gShootCount1000+48,gShootCount100+48,' ',gShootCount1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(4, gShootCount10+48);	// lets the character blink
		}

/************* SHOOT COUNT 100 ********************/  //TK: for faster changeing the 10th
		else if (state == ST_SHOOT_COUNT_SET_100)
		{
			if (input == KEY_PLUS)
				gShootCount100++;
			if (input == KEY_MINUS)
				gShootCount100--;

			if (gShootCount100 > 9)		// TK: overflow handling
				gShootCount100 = 0;

			if (gShootCount100 < 0)		// TK: Underflow handling
 				gShootCount100 = 9;

			if (input == KEY_NEXT)			// TK: go to change the 10s
			{	state = ST_SHOOT_COUNT_SET_10;
			}

			if (input == KEY_PREV)			// TK: go to change the 1000s
			{	state = ST_SHOOT_COUNT_SET_1000;
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_COUNT;
				SaveConfig();	// Save the whole configuration
			}


			DispString('C',' ',gShootCount1000+48,' ',gShootCount10+48,gShootCount1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(3, gShootCount100+48);	// lets the character blink

		}

/************* SHOOT COUNT 1000 ********************/  //TK: for faster changeing the 10th
		else if (state == ST_SHOOT_COUNT_SET_1000)
		{
			if (input == KEY_PLUS)
				gShootCount1000++;
			if (input == KEY_MINUS)
				gShootCount1000--;

			if (gShootCount1000 > 9)		// TK: overflow handling
				gShootCount1000 = 0;

			if (gShootCount1000 < 0)		// TK: Underflow handling
 				gShootCount1000 = 9;

			if (input == KEY_NEXT)			// TK: go to change the 100s
				state = ST_SHOOT_COUNT_SET_100;

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_COUNT;
				SaveConfig();	// Save the whole configuration
			}



			DispString('C',' ',' ',gShootCount100+48,gShootCount10+48,gShootCount1+48);	// Displays the string
			// +48 needet to have the ASCII sign of 0
			BlinkingCharacter(2, gShootCount1000+48);	// lets the character blink
		}

/************* DRI_SET_MIN ********************/
		else if (state == ST_DRI_SET_MIN)
		{
		    LCD_puts_f(PSTR("D MIN "),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SHOOT_COUNT;
			if (input == KEY_MINUS)
				state = ST_DRI_SET_MAX;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_DRI_SHUTTER_MIN;
			
		}

/************* DRI MIN Settings ********************/  //TK: for DRI functions
		else if (state == ST_DRI_SHUTTER_MIN)
		{
			if (input == KEY_PLUS)
				gDRIMin += gDRIStep;
			if (input == KEY_MINUS)
				gDRIMin -= gDRIStep;

			if (gDRIMin >= NumberOfTimes)
				gDRIMin = 0;
			if (gDRIMin < 0)
				gDRIMin = NumberOfTimes - gDRIStep;

			if (input == KEY_ENTER)
			{
				state = ST_DRI_SET_MIN;
				SaveConfig();	// Save the whole configuration
			}

			LCD_puts(pShutterSpeed[gDRIMin],0);

		}

/************* DRI_SET_MAX ********************/
		else if (state == ST_DRI_SET_MAX)
		{
		    LCD_puts_f(PSTR("D MAX "),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_DRI_SET_MIN;
			if (input == KEY_MINUS)
				state = ST_DRI_SET_STEP;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_DRI_SHUTTER_MAX;
			
		}

/************* DRI MAX Settings ********************/  //TK: for DRI functions
		else if (state == ST_DRI_SHUTTER_MAX)
		{
			if (input == KEY_PLUS)
				gDRIMax += gDRIStep;
			if (input == KEY_MINUS)
				gDRIMax -= gDRIStep;

			if (gDRIMax >= NumberOfTimes)
				gDRIMax = 0;
			if (gDRIMax < 0)
				gDRIMax = NumberOfTimes - gDRIStep;


			if (input == KEY_ENTER)
			{
				state = ST_DRI_SET_MAX;
				SaveConfig();	// Save the whole configuration
			}

			LCD_puts(pShutterSpeed[gDRIMax],0);
	
		}


/************* DRI_STEP ********************/
		else if (state == ST_DRI_SET_STEP)
		{
		    LCD_puts_f(PSTR("D STEP"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_DRI_SET_MAX;
			if (input == KEY_MINUS)
				state = ST_MIRROR_TIME;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_DRI_SHUTTER_STEP;
			
		}

/************* DRI STEP Settings ********************/  //FJB: for DRI Aperture Step
		else if (state == ST_DRI_SHUTTER_STEP)
		{
			if ((input == KEY_PLUS) || (input == KEY_MINUS))
			{
				if ( gDRIStep == 1 )
					gDRIStep = 3;
				else
					gDRIStep = 1;
			}

			if (input == KEY_ENTER)
			{
				state = ST_DRI_SET_STEP;
				SaveConfig();	// Save the whole configuration
			}	

			if ( gDRIStep == 1 )
				LCD_puts_f(PSTR("F  1/3"),0);
			else
				LCD_puts_f(PSTR("F FULL"),0);
	}


/************* MIRROR TIME ********************/	// FJB: Time to lock Mirror
		else if (state == ST_MIRROR_TIME)
		{
		    LCD_puts_f(PSTR("MIRROR"),0);			// output string to LCD

			if (input == KEY_PLUS)
				state = ST_DRI_SET_STEP;
			if (input == KEY_MINUS)
				state = ST_CAMERA_WAIT;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_MIRROR_TIME_SET;
			
		}
/************* MIRROR TIME Settings ********************/  //FJB: Set Time to lock Mirror
		else if (state == ST_MIRROR_TIME_SET)
		{
			if (input == KEY_PLUS)
				gMirrorTime++;
			if (input == KEY_MINUS)
				gMirrorTime--;

			if (gMirrorTime >= NumberOfMirror)
				gMirrorTime = 0;
			if (gMirrorTime < 0)
				gMirrorTime = NumberOfMirror - 1;

			if (input == KEY_ENTER)
			{
				state = ST_MIRROR_TIME;
				SaveConfig();						// Save the whole configuration
			}

			LCD_puts(pMirrorTime[gMirrorTime],0);	// Displays the string
		}


/************* CAMERA WAIT ********************/	// FJB: Time to wait for Camera
		else if (state == ST_CAMERA_WAIT)
		{
		    LCD_puts_f(PSTR("CAM WT"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_MIRROR_TIME;
			if (input == KEY_MINUS)
				state = ST_SHOOT_TIME;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_CAMERA_WAIT_SET;
			
		}
/************* CAMERA WAIT Settings ********************/  //FJB: Set Time to wait for Camera
		else if (state == ST_CAMERA_WAIT_SET)
		{
			if (input == KEY_PLUS)
				gCameraWait++;
			if (input == KEY_MINUS)
				gCameraWait--;

			if (gCameraWait >= NumberOfCamera)
				gCameraWait = 0;
			if (gCameraWait < 0)
				gCameraWait = NumberOfCamera - 1;

			if (input == KEY_ENTER)
			{
				state = ST_CAMERA_WAIT;
				SaveConfig();						// Save the whole configuration
			}

			LCD_puts(pCameraWait[gCameraWait],0);	// Displays the string
		}




//// Submenues for OPTION-part
/************* OPTION BUZZER ******************/
		else if (state == ST_OPTION_BUZZER)
		{
		    LCD_puts_f(PSTR("BUZZER"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_BUZZER_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_LED;

			if (input == KEY_PLUS)
				state = ST_OPTION_CLICK;
		}
/************* OPTION BUZZER SET **************/
		else if (state == ST_OPTION_BUZZER_SET)
		{

			if (gBuzzer)
		    	LCD_puts_f(PSTR("ON"),0);
			else
		    	LCD_puts_f(PSTR("OFF"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_BUZZER;
				eeprom_write_byte_169(E2PROM_BUZZER,gBuzzer);	// store the buzzer setting in EEPROM
			}

			if ( (input == KEY_PLUS) || (input == KEY_MINUS) )
				gBuzzer ^= 1;	// toggle the buzzer setting
		}
/************* OPTION CLICK *******************/
		else if (state == ST_OPTION_CLICK)
		{
		    LCD_puts_f(PSTR("CLICK"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_CLICK_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_BUZZER;

			if (input == KEY_PLUS)
				state = ST_OPTION_LED;
		}
/************* OPTION CLICK SET ***************/
		else if (state == ST_OPTION_CLICK_SET)
		{

			if (gClick)
		    	LCD_puts_f(PSTR("ON"),0);
			else
		    	LCD_puts_f(PSTR("OFF"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_CLICK;
				KeyClickStatus= gClick;		// update the status
				eeprom_write_byte_169(E2PROM_CLICK,gClick);	// store the click setting in EEPROM
			}

			if ( (input == KEY_PLUS) || (input == KEY_MINUS) )
				gClick ^= 1;	// toggle the click setting
		}

/************* OPTION LED *********************/
		else if (state == ST_OPTION_LED)
		{
		    LCD_puts_f(PSTR("LED"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_LED_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_CLICK;

			if (input == KEY_PLUS)

				state = ST_OPTION_BUZZER;
		}
/************* OPTION LED SET *****************/
		else if (state == ST_OPTION_LED_SET)
		{

			if (gLED == LED_STATE_OFF)
		    	LCD_puts_f(PSTR("OFF"),0);
			else if (gLED == LED_STATE_ON)
		    	LCD_puts_f(PSTR("ON"),0);
			else if (gLED == LED_STATE_LIGHT)
		    	LCD_puts_f(PSTR("LIGHT"),0);
			else if (gLED == LED_STATE_AUTO)
		    	LCD_puts_f(PSTR("AUTO"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_LED;
				eeprom_write_byte_169(E2PROM_LED,gLED);	// store the LED setting in EEPROM
			}

			if (input == KEY_MINUS)
			{
				gLED ++;
				if (gLED > LED_STATE_AUTO)
					gLED = LED_STATE_OFF;
			}
			if (input == KEY_PLUS)
			{
				gLED --;
				if (gLED < LED_STATE_OFF)
					gLED = LED_STATE_AUTO;
			}
		}

        LCD_UpdateRequired(TRUE, 0);
		ADC_value_backup = ADC_value;
	}
	
	
	return 0; // mt 
}



/*****************************************************************************
*
*   Function name : Initialization
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Initializate the different modules
*
*****************************************************************************/
void Initialization(void)
{
//    char tst;           // dummy

    OSCCAL_calibration();       // calibrate the OSCCAL byte
        
    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable

    // set prescaler = 8, Inter RC 8Mhz / 8 = 1Mhz
    CLKPR = (1<<CLKPS1) | (1<<CLKPS0);

    // Disable Analog Comparator (power save)
    ACSR = (1<<ACD);

    // Disable Digital input on PF0-2 (power save)
    DIDR1 = (7<<ADC0D);

    // mt PORTB = (15<<PORTB0);       // Enable pullup on 
	PORTB = (15<<PB0);       // Enable pullup on 
    // mt PORTE = (15<<PORTE4);
	PORTE = (15<<PE4);

    sbiBF(DDRB, 5);               // set OC1A as output
    sbiBF(PORTB, 5);              // set OC1A high
            
    Button_Init();              // Initialize pin change interrupt on joystick
    
    RTC_init();                 // Start timer2 asynchronous, used for RTC clock

    Timer0_Init();              // Used when playing music etc.

    //USART_Init(12);             // Baud rate = 9600bps
    
    //DF_SPI_init();              // init the SPI interface to communicate with the DataFlash
    
    //tst = Read_DF_status();

    //DF_CS_inactive;             // disable DataFlash
        
    LCD_Init();                 // initialize the LCD
}










/*****************************************************************************
*
*   Function name : Delay
*
*   Returns :       None
*
*   Parameters :    unsigned long millisec
*
*   Purpose :       Delay-loop
*
*****************************************************************************/
void Delay(unsigned long millisec)
{
    // mt, int i did not work in the simulator:  int i; 
	uint8_t i;
    
    while (millisec--)
        	//mt: for (i=0; i<125; i++);
			for (i=0; i<192; i++)  
				asm volatile ("nop"::);
}




/*****************************************************************************
*
*   Function name : OSCCAL_calibration
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Calibrate the internal OSCCAL byte, using the external 
*                   32,768 kHz crystal as reference
*
*****************************************************************************/
void OSCCAL_calibration(void)
{
    unsigned char calibrate = FALSE;
    int temp;
    unsigned char tempL;

    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable
    // set prescaler = 8, Inter RC 8Mhz / 8 = 1Mhz
    CLKPR = (1<<CLKPS1) | (1<<CLKPS0);
    
    TIMSK2 = 0;             //disable OCIE2A and TOIE2

    ASSR = (1<<AS2);        //select asynchronous operation of timer2 (32,768kHz)
    
    OCR2A = 200;            // set timer2 compare value 

    TIMSK0 = 0;             // delete any interrupt sources
        
    TCCR1B = (1<<CS10);     // start timer1 with no prescaling
    TCCR2A = (1<<CS20);     // start timer2 with no prescaling

    while((ASSR & 0x01) | (ASSR & 0x04));       //wait for TCN2UB and TCR2UB to be cleared

    Delay(1000);    // wait for external crystal to stabilise
    
    while(!calibrate)
    {
        cli(); // mt __disable_interrupt();  // disable global interrupt
        
        TIFR1 = 0xFF;   // delete TIFR1 flags
        TIFR2 = 0xFF;   // delete TIFR2 flags
        
        TCNT1H = 0;     // clear timer1 counter
        TCNT1L = 0;
        TCNT2 = 0;      // clear timer2 counter
           
	// shc/mt while ( !(TIFR2 && (1<<OCF2A)) );   // wait for timer2 compareflag    
        while ( !(TIFR2 & (1<<OCF2A)) );   // wait for timer2 compareflag

        TCCR1B = 0; // stop timer1

        sei(); // __enable_interrupt();  // enable global interrupt
    
        // shc/mt if ( (TIFR1 && (1<<TOV1)) )
	if ( (TIFR1 & (1<<TOV1)) )
        {
            temp = 0xFFFF;      // if timer1 overflows, set the temp to 0xFFFF
        }
        else
        {   // read out the timer1 counter value
            tempL = TCNT1L;
            temp = TCNT1H;
            temp = (temp << 8);
            temp += tempL;
        }
    
        if (temp > 6250)
        {
            OSCCAL--;   // the internRC oscillator runs to fast, decrease the OSCCAL
        }
        else if (temp < 6120)
        {
            OSCCAL++;   // the internRC oscillator runs to slow, increase the OSCCAL
        }
        else
            calibrate = TRUE;   // the interRC is correct
    
        TCCR1B = (1<<CS10); // start timer1
    }
}


/*****************************************************************************
*
*   Function name : SaveConfig
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Saves the configuration made in SET-Menu
*
*****************************************************************************/

void SaveConfig(void)
{
				// Store the Shoot-Time
				eeprom_write_byte_169(E2PROM_SHOOTSEC_1,gShootSec1);	// store the changed setting in EEPROM
				eeprom_write_byte_169(E2PROM_SHOOTSEC_10,gShootSec10);
				eeprom_write_byte_169(E2PROM_SHOOTMIN_1,gShootMin1);
				eeprom_write_byte_169(E2PROM_SHOOTMIN_10,gShootMin10);

				// Store the Wait-Time
				eeprom_write_byte_169(E2PROM_WAITSEC_1,gWaitSec1);	// store the changed setting in EEPROM
				eeprom_write_byte_169(E2PROM_WAITSEC_10,gWaitSec10);
				eeprom_write_byte_169(E2PROM_WAITMIN_1,gWaitMin1);
				eeprom_write_byte_169(E2PROM_WAITMIN_10,gWaitMin10);				
				
				// Store the Counts of images
				eeprom_write_byte_169(E2PROM_SHOOTCOUNT_1,gShootCount1);	// store the changed setting in EEPROM
				eeprom_write_byte_169(E2PROM_SHOOTCOUNT_10,gShootCount10);
				eeprom_write_byte_169(E2PROM_SHOOTCOUNT_100,gShootCount100);
				eeprom_write_byte_169(E2PROM_SHOOTCOUNT_1000,gShootCount1000);

				// Store the DRI Settings
				eeprom_write_byte_169(E2PROM_DRI_MIN,gDRIMin);
				eeprom_write_byte_169(E2PROM_DRI_MAX,gDRIMax);
				eeprom_write_byte_169(E2PROM_DRI_STEP,gDRIStep);

				// FJB: Store the Mirror and Camera Settings
				eeprom_write_byte_169(E2PROM_MIRROR_TIME, gMirrorTime);
				eeprom_write_byte_169(E2PROM_CAMERA_WAIT, gCameraWait);

}


/*****************************************************************************
*
*   Function name : Blinking Character
*
*   Returns :       None
*
*   Parameters :    PlaceNumber (The Number of characters on LCD from left to right
*					character (The character that will be displayed)
*
*	Globals:		i for counting
*					blinktime (how fast the character will be blinking, 1= fast, 4=slow)
*
*   Purpose :       Generates a blinking character at the placenumber (to be used instad of a curser)
*
*****************************************************************************/

void BlinkingCharacter(int PlaceNumber, char character)
{

			if (i>(2*blinktime))	// TK: blinking character to indicate where the cursor is
			{
				i=0;
			}

			if (i<blinktime)
			{
	            LCD_putc(PlaceNumber, character);
				i++;
			}else			
			{
			    LCD_putc(PlaceNumber, ' ');
				i++;
			}

}

/*****************************************************************************
*
*   Function name : DispString
*
*   Returns :       None
*
*   Parameters :    character0: character at Position 0
*					character1: character at Position 1
*					character2: character at Position 2
*					character3: character at Position 3
*					character4: character at Position 4
*					character5: character at Position 5
*
*
*   Purpose :       Displays the caracter according to their parameter place on the LCD. Function includes a '\0' at the end
*
*****************************************************************************/
void DispString(unsigned char character0, unsigned char character1, unsigned char character2, unsigned char character3, unsigned char character4, unsigned char character5)
{

            LCD_putc(0, character0);
   	        LCD_putc(1, character1);
       	    LCD_putc(2, character2);
           	LCD_putc(3, character3);
            LCD_putc(4, character4);
   	        LCD_putc(5, character5);
       	    LCD_putc(6, '\0');
}

